from ..athena.athena_read import athinput
from numpy import logspace, log10, zeros
from netCDF4 import Dataset
from .utils import get_rt_bands, get_ray_out
import re, subprocess

def create_inputs(tmpfile, args):
    with open(tmpfile, 'r') as file:
      tmpinp = file.read()

    pmax, pmin, np = tuple(map(float, args['plevel'].split(':')))
    np = int(np)
    plevel = logspace(log10(pmax), log10(pmin), np)
    plevel = ['%10.2f'%x for x in plevel]

    if args['tem'] == '0':
        Tp = ['0.']*np
    else:
        Tp = args['tem'].split(' ')
    assert(len(Tp) == np)

    if args['nh3'] == '0':
        NH3p  = ['0.']*np
    else:
        NH3p = args['nh3'].split(' ')
    assert(len(NH3p) == np)

  # adjust minimum number of walkers
    #args['nwalker'] = str(max(int(args['nwalker']), 2*np))

    var = [x for x in args['var']]

    name = tmpfile.split('.')[0]
    if args['output'] != '':
        name += '-' + args['output']

    inpfile = re.sub('\[problem_id\]', name, tmpinp)
    inpfile = re.sub('\[logname\]', name, inpfile)
    inpfile = re.sub('\[obsname\]', args['obs'], inpfile)
    inpfile = re.sub('\[T0\]', args['T0'], inpfile)
    inpfile = re.sub('\[Tmin\]', args['Tmin'], inpfile)
    inpfile = re.sub('\[qH2O\]', args['qH2O'], inpfile)
    inpfile = re.sub('\[qNH3\]', args['qNH3'], inpfile)
    inpfile = re.sub('\[Tstd\]', args['sT'], inpfile)
    inpfile = re.sub('\[Xstd\]', args['sNH3'], inpfile)
    inpfile = re.sub('\[Tlen\]', args['zT'], inpfile)
    inpfile = re.sub('\[Xlen\]', args['zNH3'], inpfile)
    inpfile = re.sub('\[plevel\]', ' '.join(plevel), inpfile)
    inpfile = re.sub('\[pmin\]', args['pmin'], inpfile)
    inpfile = re.sub('\[pmax\]', args['pmax'], inpfile)
    inpfile = re.sub('\[nwalker\]', args['nwalker'], inpfile)
    inpfile = re.sub('\[nlim\]', args['nlim'], inpfile)
    inpfile = re.sub('\[variables\]', ' '.join(var), inpfile)
    inpfile = re.sub('\[nodes\]', str(4*int(args['nodes'])), inpfile)
    inpfile = re.sub('\[Tp\]', ' '.join(Tp), inpfile)
    inpfile = re.sub('\[NH3p\]', ' '.join(NH3p), inpfile)
    inpfile = re.sub('\[grav\]', '-' + args['grav'], inpfile)
    inpfile = re.sub('\[lat\]', args['lat'], inpfile)
    inpfile = re.sub('\[rgradt\]', args['rgradt'], inpfile)
    inpfile = re.sub('\[metallicity\]', args['metallicity'], inpfile)
    inpfile = re.sub('\[karpowicz_scale\]', args['karpowicz_scale'], inpfile)
    inpfile = re.sub('\[hanley_power\]', args['hanley_power'], inpfile)
    if args['d']:
        inpfile = re.sub('\[diff\]', 'true', inpfile)
    else:
        inpfile = re.sub('\[diff\]', 'false', inpfile)

    with open(name + '.inp', 'w') as file:
        file.write(inpfile)
    print('Input file written to %s.inp\n' % name)
    return name + '.inp'

def run_forward(exefile, inpfile):
    script = ['./' + exefile, '-i', inpfile]
    process = subprocess.Popen(script,
        stdout = subprocess.PIPE,
        stderr = subprocess.PIPE)
    while True:
        output = process.stdout.readline()
        #err = process.stderr.readline()
        #if (err != ''):
        #  raise Exception(err.decode('UTF-8'))
        if output == b'' and process.poll() is not None:
            break
        if output:
            print(output.decode('UTF-8'), end = '')
    process.poll()
    #print(out.decode('UTF-8'), end = '\r')
    #print(err.decode('UTF-8'))

    out, err = subprocess.Popen('./combine.py',
        stdout = subprocess.PIPE,
        stderr = subprocess.PIPE).communicate()
    print(out.decode('UTF-8'), end = '')
    print(err.decode('UTF-8'), end = '')

    inp = athinput(inpfile)
    return inp['job']['problem_id']

def write_observation(inpfile, datafile):
    freq = get_rt_bands(inpfile)[:,0]
    num_bands = len(freq)
    amu, aphi = get_ray_out(inpfile)
    num_dirs = len(amu)

# read radiation toa
#datafile = 'vla_ideal_js-main.nc'
    data = Dataset(datafile, 'r')
    tb = zeros((4, num_bands, num_dirs))
    for i in range(num_bands):
        for j in range(num_dirs):
            tb[0,i,j] = data['b%dtoa%d' % (i+1,j+1)][0,0,0]
            tb[1,i,j] = data['b%dtoa%d' % (i+1,j+1)][0,1,0]
            tb[2,i,j] = data['b%dtoa%d' % (i+1,j+1)][0,2,0]
            tb[3,i,j] = data['b%dtoa%d' % (i+1,j+1)][0,3,0]

# write to file
    outfile = '.'.join(inpfile.split('.')[:-1]) + '.out'
    with open(outfile, 'w') as file:
        file.write('# Brightness temperatures of input model %s - model 0\n' % inpfile)
        file.write('%12s' % '# Freq (GHz)')
        for i in range(num_dirs):
            file.write('%10.1f' % amu[i])
        file.write('\n')
        for i in range(num_bands):
            file.write('%12.1f' % freq[i])
            for j in range(num_dirs):
                file.write('%10.2f' % tb[0,i,j])
            file.write('\n')

        file.write('# Brightness temperatures of input model %s - model 1\n' % inpfile)
        file.write('%12s' % '# Freq (GHz)')
        for i in range(num_dirs):
            file.write('%10.1f' % amu[i])
        file.write('\n')
        for i in range(num_bands):
            file.write('%12.1f' % freq[i])
            for j in range(num_dirs):
                file.write('%10.2f' % tb[1,i,j])
            file.write('\n')

        file.write('# Brightness temperatures of input model %s - model 2\n' % inpfile)
        file.write('%12s' % '# Freq (GHz)')
        for i in range(num_dirs):
            file.write('%10.1f' % amu[i])
        file.write('\n')
        for i in range(num_bands):
            file.write('%12.1f' % freq[i])
            for j in range(num_dirs):
                file.write('%10.2f' % tb[2,i,j])
            file.write('\n')

        file.write('# Brightness temperatures of input model %s - model 3\n' % inpfile)
        file.write('%12s' % '# Freq (GHz)')
        for i in range(num_dirs):
            file.write('%10.1f' % amu[i])
        file.write('\n')
        for i in range(num_bands):
            file.write('%12.1f' % freq[i])
            for j in range(num_dirs):
                file.write('%10.2f' % tb[3,i,j])
            file.write('\n')
    print('Brightness temperatures written to %s' % outfile)
    return outfile



import argparse

parser = argparse.ArgumentParser()
parser.add_argument('-d',
    action = 'store_true',
    default = False,
    help = 'fit differential observation'
    )
parser.add_argument('-i', '--input',
    required = True,
    help = 'template input file (required)'
    )
parser.add_argument('-o', '--output',
    default = '',
    help = 'output id'
    )
parser.add_argument('--qNH3',
    default = '2.7',
    help = 'ammonia abundance'
    )
parser.add_argument('--qH2O',
    default = '20.',
    help = 'water abundance'
    )
parser.add_argument('--sNH3',
    default = '0.5',
    help = 'ammonia standard deviation'
    )
parser.add_argument('--sT',
    default = '5.',
    help = 'temperature standard deviation'
    )
parser.add_argument('--zNH3',
    default = '20',
    help = 'ammonia correlation'
    )
parser.add_argument('--zT',
    default = '20',
    help = 'temperature correlation'
    )
parser.add_argument('--T0',
    default = '166',
    help = 'reference temperature'
    )
parser.add_argument('--grav',
    default = '10.',
    help = 'surface gravity'
    )
parser.add_argument('--Tmin',
    default = '110',
    help = 'minimum temperature'
    )
parser.add_argument('--nodes',
    default = '1',
    help = 'number of nodes to use'
    )
parser.add_argument('--plevel',
    default = '50:0.6:5',
    help = 'pressure divides'
    )
parser.add_argument('--pmin',
    default = '0.3',
    help = 'minimum pressure'
    )
parser.add_argument('--pmax',
    default = '100.',
    help = 'maximum pressure'
    )
parser.add_argument('--tem',
    default = '0',
    help = 'temperature anomaly'
    )
parser.add_argument('--nh3',
    default = '0',
    help = 'ammonia anomaly'
    )
parser.add_argument('--var',
    default = '2',
    help = 'inversion variables'
    )
parser.add_argument('--nlim',
    default = '0',
    help = 'number of MCMC steps'
    )
parser.add_argument('--nwalker',
    default = '1',
    help = 'number of mcmc walkers'
    )
parser.add_argument('--obs',
    default = 'none',
    help = 'observation file'
    )
parser.add_argument('--lat',
    default = '0.',
    help = 'latitude'
    )
parser.add_argument('--rgradt',
    default = '1.',
    help = 'radiative temperature gradient'
    )
parser.add_argument('--metallicity',
    default = '0.',
    help = 'metallicity applied to Na/KCl'
    )
parser.add_argument('--karpowicz_scale',
    default = '0.',
    help = 'scale karpowicz water opacity model'
    )
parser.add_argument('--hanley_power',
    default = '0.',
    help = 'scale hanley ammonia opacity model'
    )
args = vars(parser.parse_args())
if '.' not in args['input']:
    args['input'] += '.tmp'



#!/usr/bin/env python3

import os, sysconfig
from glob import glob
from setuptools import setup, find_packages, Extension

# https://cython.readthedocs.io/en/latest/src/userguide/source_files_and_compilation.html#distributing-cython-modules
def no_cythonize(extensions, **_ignore):
  for extension in extensions:
    sources = []
    for sfile in extension.sources:
      path, ext = os.path.splitext(sfile)
      if ext in (".pyx", ".py"):
        if extension.language == "c++":
            ext = ".cpp"
        else:
            ext = ".c"
        sfile = path + ext
      sources.append(sfile)
    extension.sources[:] = sources
  return extensions

athena_dir = '../src/'
srcs = ['parameter_input.cpp', 
        'globals.cpp',
        'outputs/io_wrapper.cpp',
        'utils/utils.cpp']
srcs = [athena_dir + x for x in srcs]
srcs += glob(athena_dir + 'radiation/*.cpp')

extra_compile_args = sysconfig.get_config_var('CFLAGS').split()
extra_compile_args += ['-std=c++11']

pyharp = Extension('athenapp.pyharp',
  define_macros = [('MAJOR_VERSION', '0'),
                   ('MINOR_VERSION', '1')],
  include_dirs = [],
  libraries = ['netcdf'],
  #library_dirs = ['/usr/local/lib'],
  sources = ['athenapp/pyharp/pyharp.pyx'] + srcs,
  extra_compile_args = extra_compile_args,
  language = 'c++'
  )

pyathena = Extension('athenapp.pyathena',
  define_macros = [('MAJOR_VERSION', '0'),
                   ('MINOR_VERSION', '1')],
  include_dirs = [],
  libraries = [],
  #library_dirs = ['/usr/local/lib'],
  sources = ['athenapp/pyathena/pyathena.pyx'],
  extra_compile_args = extra_compile_args,
  language = 'c++'
  )

extensions = [pyathena, pyharp]

CYTHONIZE = bool(int(os.getenv("CYTHONIZE", 0)))

if CYTHONIZE:
  compiler_directives = {"language_level": 2, "embedsignature": True}
  extensions = Cython.Build.cythonize(extensions, compiler_directives = compiler_directives)
else:
  extensions = no_cythonize(extensions)

with open("requirements.txt") as fp:
  install_requires = fp.read().strip().split("\n")

with open("requirements-dev.txt") as fp:
  dev_requires = fp.read().strip().split("\n")
  
with open("README", "r") as fh:
  long_description = fh.read()

setup(
  name="snapy", # Replace with your own username
  version="0.0.4",
  author="Cheng Li",
  author_email="chengcli@umich.edu",
  description="python extension for SNAP model",
  long_description=long_description,
  long_description_content_type="text/markdown",
  url="https://snap.chengcli.io",
  #ext_modules = extensions,
  ext_modules = None,
  install_requires = install_requires,
  extras_require = {
    "dev": dev_requires,
    "docs": ["sphinx", "sphinx-rtd-theme"]
  },
  packages = ['snapy', 'snapy.athena', 'snapy.harp'],
  #packages = find_packages(),
  classifiers = [
      "Programming Language :: Python :: 3",
      "License :: OSI Approved :: MIT License",
      "Operating System :: OS Independent",
  ],
  python_requires='>=3.0',
)



from numpy import array
from ..athena.athena_read import athinput
import re

def get_rt_bands(inpfile):
  inp = athinput(inpfile)

  # count number of bands
  num_bands = 0
  for key in inp['radiation'].keys():
    if re.match('b[0-9]+$', key):
      num_bands += 1

  # read frequency
  band_info = []
  for i in range(num_bands):
    band_info.append(list(map(float, inp['radiation']['b%d' % (i+1)].split())))
  band_info = array(band_info)

  return band_info

def get_ray_out(inpfile):
  inp = athinput(inpfile)

  # out directions
  outdir = inp['radiation']['outdir'].split(' ')
  num_dirs = len(outdir)

  # polar and azimuthal angle
  amu, aphi = [], []
  for i in range(num_dirs):
    m = re.search('\((.*),(.*)\)', outdir[i])
    if m.group(1) != '':
      amu.append(float(m.group(1)))
    else:
      amu.append(0.)
    if m.group(2) != '':
      aphi.append(float(m.group(2)))
    else:
      aphi.append(0.)
  return array(amu), array(aphi)

def get_sample_pressure(inpfile):
  inp = athinput(inpfile)
  return list(map(float, inp['inversion']['PrSample'].split()))
